#pragma once

#include "messagebasic_global.h"
#include <simsbasic.h>
#include <standardglobe.h>
#include <QStringList>
#include <QHash>

#define NAME(v) #v
#define DOUBLE_SAVE(u) obj[NAME(u)] = u
#define DOUBLE_PEAK(u) u = obj[NAME(u)].toDouble()

#define UINT64_SAVE(u) obj[NAME(u)] = QJsonValue::fromVariant(QVariant::fromValue(u))
#define UINT64_PEAK(u) u = obj[NAME(u)].toVariant().toULongLong()

#define STRING_SAVE(u) obj[NAME(u)] = u
#define STRING_PEAK(u) u = obj[NAME(u)].toString()

#define STRLIST_SAVE(u) obj[NAME(u)] = u.join(",")
#define STRLIST_PEAK(u) u = obj[NAME(u)].toString().split(",")

class MESSAGEBASIC_EXPORT AbstractMessage : public WsMessage {
protected:
	uint64_t _from_id = 0, _to_id = 0;
	QString _topic_string;

	AbstractMessage(const QString& topic);

public:
	virtual ~AbstractMessage() = default;

	void reset(uint64_t from, uint64_t to);

	// ͨ WsMessage ̳
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
	QString topicString() const override;
	uint64_t targetEntity() const override;
	uint64_t sourceEntity() const override;
};

/// <summary>
/// ʱͬϢ
/// </summary>
struct MESSAGEBASIC_EXPORT SyncRequest : public AbstractMessage {
	double _time_current;

	SyncRequest();

	// ͨ WsMessage ̳
	std::shared_ptr<TopicData> newDefault() const override;
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

/// <summary>
/// 
/// </summary>
struct MESSAGEBASIC_EXPORT DeduceRequest : public AbstractMessage {
	DeduceRequest();
	std::shared_ptr<TopicData> newDefault() const override;
};

/// <summary>
/// Ĭϵ޷ϢĴͨ÷Ϣ
/// </summary>
struct MESSAGEBASIC_EXPORT RespondDefault : public AbstractMessage {
	double _time_consume;
	bool _success_mark = false;
	QString _reason_text = "";

	RespondDefault();

	// ͨ WsMessage ̳
	std::shared_ptr<TopicData> newDefault() const override;
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};
